/*!-----------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Version: 0.2.1(0e81224179fbb8f6fda18ca7362d8500a263cfef)
 * Released under the MIT license
 * https://github.com/Microsoft/vscode/blob/master/LICENSE.txt
 *-----------------------------------------------------------*/

define("vs/languages/typescript/common/tokenization", ["require", "exports", 'vs/languages/typescript/common/lib/typescriptServices'], function (require, exports, ts) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Language) {
        Language[Language["TypeScript"] = 0] = "TypeScript";
        Language[Language["EcmaScript5"] = 1] = "EcmaScript5";
    })(exports.Language || (exports.Language = {}));
    var Language = exports.Language;
    function createTokenizationSupport2(language) {
        var classifier = ts.createClassifier(), bracketTypeTable = language === Language.TypeScript ? tsBracketTypeTable : jsBracketTypeTable, tokenTypeTable = language === Language.TypeScript ? tsTokenTypeTable : jsTokenTypeTable;
        return {
            getInitialState: function () { return new State2(language, ts.EndOfLineState.None, false); },
            tokenize: function (line, state) { return tokenize2(bracketTypeTable, tokenTypeTable, classifier, state, line); }
        };
    }
    exports.createTokenizationSupport2 = createTokenizationSupport2;
    var State2 = (function () {
        function State2(language, eolState, inJsDocComment) {
            this.language = language;
            this.eolState = eolState;
            this.inJsDocComment = inJsDocComment;
        }
        State2.prototype.clone = function () {
            return new State2(this.language, this.eolState, this.inJsDocComment);
        };
        State2.prototype.equals = function (other) {
            if (other === this) {
                return true;
            }
            if (!other || !(other instanceof State2)) {
                return false;
            }
            if (this.eolState !== other.eolState) {
                return false;
            }
            if (this.inJsDocComment !== other.inJsDocComment) {
                return false;
            }
            return true;
        };
        return State2;
    }());
    function tokenize2(bracketTypeTable, tokenTypeTable, classifier, state, text) {
        // Create result early and fill in tokens
        var ret = {
            tokens: [],
            endState: new State2(state.language, ts.EndOfLineState.None, false)
        };
        function appendFn(startIndex, type) {
            if (ret.tokens.length === 0 || ret.tokens[ret.tokens.length - 1].scopes !== type) {
                ret.tokens.push({
                    startIndex: startIndex,
                    scopes: type
                });
            }
        }
        var isTypeScript = state.language === Language.TypeScript;
        // shebang statement, #! /bin/node
        if (!isTypeScript && checkSheBang(0, text, appendFn)) {
            return ret;
        }
        var result = classifier.getClassificationsForLine(text, state.eolState, true), offset = 0;
        ret.endState.eolState = result.finalLexState;
        ret.endState.inJsDocComment = result.finalLexState === ts.EndOfLineState.InMultiLineCommentTrivia && (state.inJsDocComment || /\/\*\*.*$/.test(text));
        for (var _i = 0, _a = result.entries; _i < _a.length; _i++) {
            var entry = _a[_i];
            var type;
            if (entry.classification === ts.TokenClass.Punctuation) {
                // punctions: check for brackets: (){}[]
                var ch = text.charCodeAt(offset);
                type = bracketTypeTable[ch] || tokenTypeTable[entry.classification];
                appendFn(offset, type);
            }
            else if (entry.classification === ts.TokenClass.Comment) {
                // comments: check for JSDoc, block, and line comments
                if (ret.endState.inJsDocComment || /\/\*\*.*\*\//.test(text.substr(offset, entry.length))) {
                    appendFn(offset, isTypeScript ? 'comment.doc.ts' : 'comment.doc.js');
                }
                else {
                    appendFn(offset, isTypeScript ? 'comment.ts' : 'comment.js');
                }
            }
            else {
                // everything else
                appendFn(offset, tokenTypeTable[entry.classification] || '');
            }
            offset += entry.length;
        }
        return ret;
    }
    var tsBracketTypeTable = Object.create(null);
    tsBracketTypeTable['('.charCodeAt(0)] = 'delimiter.parenthesis.ts';
    tsBracketTypeTable[')'.charCodeAt(0)] = 'delimiter.parenthesis.ts';
    tsBracketTypeTable['{'.charCodeAt(0)] = 'delimiter.bracket.ts';
    tsBracketTypeTable['}'.charCodeAt(0)] = 'delimiter.bracket.ts';
    tsBracketTypeTable['['.charCodeAt(0)] = 'delimiter.array.ts';
    tsBracketTypeTable[']'.charCodeAt(0)] = 'delimiter.array.ts';
    var tsTokenTypeTable = Object.create(null);
    tsTokenTypeTable[ts.TokenClass.Identifier] = 'identifier.ts';
    tsTokenTypeTable[ts.TokenClass.Keyword] = 'keyword.ts';
    tsTokenTypeTable[ts.TokenClass.Operator] = 'delimiter.ts';
    tsTokenTypeTable[ts.TokenClass.Punctuation] = 'delimiter.ts';
    tsTokenTypeTable[ts.TokenClass.NumberLiteral] = 'number.ts';
    tsTokenTypeTable[ts.TokenClass.RegExpLiteral] = 'regexp.ts';
    tsTokenTypeTable[ts.TokenClass.StringLiteral] = 'string.ts';
    var jsBracketTypeTable = Object.create(null);
    jsBracketTypeTable['('.charCodeAt(0)] = 'delimiter.parenthesis.js';
    jsBracketTypeTable[')'.charCodeAt(0)] = 'delimiter.parenthesis.js';
    jsBracketTypeTable['{'.charCodeAt(0)] = 'delimiter.bracket.js';
    jsBracketTypeTable['}'.charCodeAt(0)] = 'delimiter.bracket.js';
    jsBracketTypeTable['['.charCodeAt(0)] = 'delimiter.array.js';
    jsBracketTypeTable[']'.charCodeAt(0)] = 'delimiter.array.js';
    var jsTokenTypeTable = Object.create(null);
    jsTokenTypeTable[ts.TokenClass.Identifier] = 'identifier.js';
    jsTokenTypeTable[ts.TokenClass.Keyword] = 'keyword.js';
    jsTokenTypeTable[ts.TokenClass.Operator] = 'delimiter.js';
    jsTokenTypeTable[ts.TokenClass.Punctuation] = 'delimiter.js';
    jsTokenTypeTable[ts.TokenClass.NumberLiteral] = 'number.js';
    jsTokenTypeTable[ts.TokenClass.RegExpLiteral] = 'regexp.js';
    jsTokenTypeTable[ts.TokenClass.StringLiteral] = 'string.js';
    function checkSheBang(deltaOffset, line, appendFn) {
        if (line.indexOf('#!') === 0) {
            appendFn(deltaOffset, 'comment.shebang');
            return true;
        }
    }
});

define("vs/languages/typescript/common/typescript", ["require", "exports", 'vs/base/common/event', 'vs/base/common/errors', 'vs/languages/typescript/common/lib/typescriptServices'], function (require, exports, event_1, errors_1, ts) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LanguageServiceDefaults = (function () {
        function LanguageServiceDefaults(compilerOptions, diagnosticsOptions) {
            this._onDidChange = new event_1.Emitter();
            this._extraLibs = Object.create(null);
            this.setCompilerOptions(compilerOptions);
            this.setDiagnosticsOptions(diagnosticsOptions);
        }
        Object.defineProperty(LanguageServiceDefaults.prototype, "onDidChange", {
            get: function () {
                return this._onDidChange.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(LanguageServiceDefaults.prototype, "extraLibs", {
            get: function () {
                return Object.freeze(this._extraLibs);
            },
            enumerable: true,
            configurable: true
        });
        LanguageServiceDefaults.prototype.addExtraLib = function (content, filePath) {
            var _this = this;
            if (typeof filePath === 'undefined') {
                filePath = "ts:extralib-" + Date.now();
            }
            if (this._extraLibs[filePath]) {
                throw new Error(filePath + " already a extra lib");
            }
            this._extraLibs[filePath] = content;
            this._onDidChange.fire(this);
            return {
                dispose: function () {
                    if (delete _this._extraLibs[filePath]) {
                        _this._onDidChange.fire(_this);
                    }
                }
            };
        };
        Object.defineProperty(LanguageServiceDefaults.prototype, "compilerOptions", {
            get: function () {
                return this._compilerOptions;
            },
            enumerable: true,
            configurable: true
        });
        LanguageServiceDefaults.prototype.setCompilerOptions = function (options) {
            this._compilerOptions = options || Object.create(null);
            this._onDidChange.fire(this);
        };
        Object.defineProperty(LanguageServiceDefaults.prototype, "diagnosticsOptions", {
            get: function () {
                return this._diagnosticsOptions;
            },
            enumerable: true,
            configurable: true
        });
        LanguageServiceDefaults.prototype.setDiagnosticsOptions = function (options) {
            this._diagnosticsOptions = options || Object.create(null);
            this._onDidChange.fire(this);
        };
        return LanguageServiceDefaults;
    }());
    exports.LanguageServiceDefaults = LanguageServiceDefaults;
    exports.typeScriptDefaults = new LanguageServiceDefaults({ allowNonTsExtensions: true, target: ts.ScriptTarget.Latest }, { noSemanticValidation: false, noSyntaxValidation: false });
    exports.javaScriptDefaults = new LanguageServiceDefaults({ allowNonTsExtensions: true, allowJs: true, target: ts.ScriptTarget.Latest }, { noSemanticValidation: true, noSyntaxValidation: false });
    var TypeScriptWorkerProtocol = (function () {
        function TypeScriptWorkerProtocol() {
        }
        // --- model sync
        TypeScriptWorkerProtocol.prototype.acceptNewModel = function (data) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.acceptModelChanged = function (uri, events) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.acceptRemovedModel = function (uri) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.acceptDefaults = function (options, extraLibs) {
            throw errors_1.notImplemented();
        };
        // --- language features
        TypeScriptWorkerProtocol.prototype.getSyntacticDiagnostics = function (fileName) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getSemanticDiagnostics = function (fileName) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getCompletionsAtPosition = function (uri, offset) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getCompletionEntryDetails = function (fileName, position, entry) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getSignatureHelpItems = function (fileName, position) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getQuickInfoAtPosition = function (fileName, position) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getOccurrencesAtPosition = function (fileName, position) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getDefinitionAtPosition = function (fileName, position) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getReferencesAtPosition = function (fileName, position) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getNavigationBarItems = function (fileName) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getFormattingEditsForDocument = function (fileName, options) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getFormattingEditsForRange = function (fileName, start, end, options) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getFormattingEditsAfterKeystroke = function (fileName, postion, ch, options) {
            throw errors_1.notImplemented();
        };
        TypeScriptWorkerProtocol.prototype.getEmitOutput = function (fileName) {
            throw errors_1.notImplemented();
        };
        return TypeScriptWorkerProtocol;
    }());
    exports.TypeScriptWorkerProtocol = TypeScriptWorkerProtocol;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/languages/typescript/common/languageFeatures", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/uri', 'vs/base/common/severity', 'vs/base/common/lifecycle', 'vs/editor/common/modes', 'vs/editor/common/modes/languageSelector', 'vs/languages/typescript/common/lib/typescriptServices', 'vs/base/common/async'], function (require, exports, winjs_base_1, uri_1, severity_1, lifecycle, modes, languageSelector_1, ts, async_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function register(modelService, markerService, selector, defaults, worker) {
        var disposables = [];
        disposables.push(modes.SuggestRegistry.register(selector, new SuggestAdapter(modelService, worker), true));
        disposables.push(modes.SignatureHelpProviderRegistry.register(selector, new SignatureHelpAdapter(modelService, worker), true));
        disposables.push(modes.HoverProviderRegistry.register(selector, new QuickInfoAdapter(modelService, worker), true));
        disposables.push(modes.DocumentHighlightProviderRegistry.register(selector, new OccurrencesAdapter(modelService, worker), true));
        disposables.push(modes.DefinitionProviderRegistry.register(selector, new DefinitionAdapter(modelService, worker), true));
        disposables.push(modes.ReferenceProviderRegistry.register(selector, new ReferenceAdapter(modelService, worker), true));
        disposables.push(modes.DocumentSymbolProviderRegistry.register(selector, new OutlineAdapter(modelService, worker), true));
        disposables.push(modes.DocumentRangeFormattingEditProviderRegistry.register(selector, new FormatAdapter(modelService, worker), true));
        disposables.push(modes.OnTypeFormattingEditProviderRegistry.register(selector, new FormatOnTypeAdapter(modelService, worker), true));
        disposables.push(new DiagnostcsAdapter(defaults, selector, markerService, modelService, worker));
        return lifecycle.combinedDisposable(disposables);
    }
    exports.register = register;
    var Adapter = (function () {
        function Adapter(_modelService, _worker) {
            this._modelService = _modelService;
            this._worker = _worker;
        }
        Adapter.prototype._positionToOffset = function (resource, position) {
            var model = this._modelService.getModel(resource);
            var result = position.column - 1;
            for (var i = 1; i < position.lineNumber; i++) {
                result += model.getLineContent(i).length + model.getEOL().length;
            }
            return result;
        };
        Adapter.prototype._offsetToPosition = function (resource, offset) {
            var model = this._modelService.getModel(resource);
            var lineNumber = 1;
            while (true) {
                var len = model.getLineContent(lineNumber).length + model.getEOL().length;
                if (offset < len) {
                    break;
                }
                offset -= len;
                lineNumber++;
            }
            return { lineNumber: lineNumber, column: 1 + offset };
        };
        Adapter.prototype._textSpanToRange = function (resource, span) {
            var p1 = this._offsetToPosition(resource, span.start);
            var p2 = this._offsetToPosition(resource, span.start + span.length);
            var startLineNumber = p1.lineNumber, startColumn = p1.column;
            var endLineNumber = p2.lineNumber, endColumn = p2.column;
            return { startLineNumber: startLineNumber, startColumn: startColumn, endLineNumber: endLineNumber, endColumn: endColumn };
        };
        return Adapter;
    }());
    // --- diagnostics --- ---
    var DiagnostcsAdapter = (function (_super) {
        __extends(DiagnostcsAdapter, _super);
        function DiagnostcsAdapter(_defaults, _selector, _markerService, modelService, worker) {
            var _this = this;
            _super.call(this, modelService, worker);
            this._defaults = _defaults;
            this._selector = _selector;
            this._markerService = _markerService;
            this._disposables = [];
            this._listener = Object.create(null);
            var onModelAdd = function (model) {
                if (!languageSelector_1.default(_selector, model.uri, model.getModeId())) {
                    return;
                }
                var handle;
                _this._listener[model.uri.toString()] = model.onDidChangeContent(function () {
                    clearTimeout(handle);
                    handle = setTimeout(function () { return _this._doValidate(model.uri); }, 500);
                });
                _this._doValidate(model.uri);
            };
            var onModelRemoved = function (model) {
                delete _this._listener[model.uri.toString()];
            };
            this._disposables.push(modelService.onModelAdded(onModelAdd));
            this._disposables.push(modelService.onModelRemoved(onModelRemoved));
            this._disposables.push(modelService.onModelModeChanged(function (event) {
                onModelRemoved(event.model);
                onModelAdd(event.model);
            }));
            this._disposables.push({
                dispose: function () {
                    for (var key in _this._listener) {
                        _this._listener[key].dispose();
                    }
                }
            });
            modelService.getModels().forEach(onModelAdd);
        }
        DiagnostcsAdapter.prototype.dispose = function () {
            this._disposables = lifecycle.dispose(this._disposables);
        };
        DiagnostcsAdapter.prototype._doValidate = function (resource) {
            var _this = this;
            this._worker(resource).then(function (worker) {
                var promises = [];
                if (!_this._defaults.diagnosticsOptions.noSyntaxValidation) {
                    promises.push(worker.getSyntacticDiagnostics(resource.toString()));
                }
                if (!_this._defaults.diagnosticsOptions.noSemanticValidation) {
                    promises.push(worker.getSemanticDiagnostics(resource.toString()));
                }
                return winjs_base_1.TPromise.join(promises);
            }).then(function (diagnostics) {
                var markers = diagnostics
                    .reduce(function (p, c) { return c.concat(p); }, [])
                    .map(function (d) { return _this._convertDiagnostics(resource, d); });
                _this._markerService.changeOne(_this._selector, resource, markers);
            }).done(undefined, function (err) {
                console.error(err);
            });
        };
        DiagnostcsAdapter.prototype._convertDiagnostics = function (resource, diag) {
            var _a = this._offsetToPosition(resource, diag.start), startLineNumber = _a.lineNumber, startColumn = _a.column;
            var _b = this._offsetToPosition(resource, diag.start + diag.length), endLineNumber = _b.lineNumber, endColumn = _b.column;
            return {
                severity: severity_1.default.Error,
                startLineNumber: startLineNumber,
                startColumn: startColumn,
                endLineNumber: endLineNumber,
                endColumn: endColumn,
                message: ts.flattenDiagnosticMessageText(diag.messageText, '\n')
            };
        };
        return DiagnostcsAdapter;
    }(Adapter));
    // --- suggest ------
    var SuggestAdapter = (function (_super) {
        __extends(SuggestAdapter, _super);
        function SuggestAdapter() {
            _super.apply(this, arguments);
        }
        Object.defineProperty(SuggestAdapter.prototype, "triggerCharacters", {
            get: function () {
                return ['.'];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SuggestAdapter.prototype, "shouldAutotriggerSuggest", {
            get: function () {
                return true;
            },
            enumerable: true,
            configurable: true
        });
        SuggestAdapter.prototype.provideCompletionItems = function (model, position, token) {
            var wordInfo = model.getWordUntilPosition(position);
            var resource = model.uri;
            var offset = this._positionToOffset(resource, position);
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getCompletionsAtPosition(resource.toString(), offset);
            }).then(function (info) {
                if (!info) {
                    return;
                }
                var suggestions = info.entries.map(function (entry) {
                    return {
                        label: entry.name,
                        codeSnippet: entry.name,
                        type: SuggestAdapter.asType(entry.kind)
                    };
                });
                return [{
                        currentWord: wordInfo && wordInfo.word,
                        suggestions: suggestions
                    }];
            }));
        };
        SuggestAdapter.prototype.resolveCompletionItem = function (model, position, suggestion, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getCompletionEntryDetails(resource.toString(), _this._positionToOffset(resource, position), suggestion.label);
            }).then(function (details) {
                if (!details) {
                    return suggestion;
                }
                return {
                    label: details.name,
                    codeSnippet: details.name,
                    type: SuggestAdapter.asType(details.kind),
                    typeLabel: ts.displayPartsToString(details.displayParts),
                    documentationLabel: ts.displayPartsToString(details.documentation)
                };
            }));
        };
        SuggestAdapter.asType = function (kind) {
            switch (kind) {
                case 'getter':
                case 'setting':
                case 'constructor':
                case 'method':
                case 'property':
                    return 'property';
                case 'function':
                case 'local function':
                    return 'function';
                case 'class':
                    return 'class';
                case 'interface':
                    return 'interface';
            }
            return 'variable';
        };
        return SuggestAdapter;
    }(Adapter));
    var SignatureHelpAdapter = (function (_super) {
        __extends(SignatureHelpAdapter, _super);
        function SignatureHelpAdapter() {
            _super.apply(this, arguments);
            this.signatureHelpTriggerCharacters = ['(', ','];
        }
        SignatureHelpAdapter.prototype.provideSignatureHelp = function (model, position, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) { return worker.getSignatureHelpItems(resource.toString(), _this._positionToOffset(resource, position)); }).then(function (info) {
                if (!info) {
                    return;
                }
                var ret = {
                    activeSignature: info.selectedItemIndex,
                    activeParameter: info.argumentIndex,
                    signatures: []
                };
                info.items.forEach(function (item) {
                    var signature = {
                        label: '',
                        documentation: null,
                        parameters: []
                    };
                    signature.label += ts.displayPartsToString(item.prefixDisplayParts);
                    item.parameters.forEach(function (p, i, a) {
                        var label = ts.displayPartsToString(p.displayParts);
                        var parameter = {
                            label: label,
                            documentation: ts.displayPartsToString(p.documentation)
                        };
                        signature.label += label;
                        signature.parameters.push(parameter);
                        if (i < a.length - 1) {
                            signature.label += ts.displayPartsToString(item.separatorDisplayParts);
                        }
                    });
                    signature.label += ts.displayPartsToString(item.suffixDisplayParts);
                    ret.signatures.push(signature);
                });
                return ret;
            }));
        };
        return SignatureHelpAdapter;
    }(Adapter));
    // --- hover ------
    var QuickInfoAdapter = (function (_super) {
        __extends(QuickInfoAdapter, _super);
        function QuickInfoAdapter() {
            _super.apply(this, arguments);
        }
        QuickInfoAdapter.prototype.provideHover = function (model, position, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getQuickInfoAtPosition(resource.toString(), _this._positionToOffset(resource, position));
            }).then(function (info) {
                if (!info) {
                    return;
                }
                return {
                    range: _this._textSpanToRange(resource, info.textSpan),
                    htmlContent: [{ text: ts.displayPartsToString(info.displayParts) }]
                };
            }));
        };
        return QuickInfoAdapter;
    }(Adapter));
    // --- occurrences ------
    var OccurrencesAdapter = (function (_super) {
        __extends(OccurrencesAdapter, _super);
        function OccurrencesAdapter() {
            _super.apply(this, arguments);
        }
        OccurrencesAdapter.prototype.provideDocumentHighlights = function (model, position, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getOccurrencesAtPosition(resource.toString(), _this._positionToOffset(resource, position));
            }).then(function (entries) {
                if (!entries) {
                    return;
                }
                return entries.map(function (entry) {
                    return {
                        range: _this._textSpanToRange(resource, entry.textSpan),
                        kind: entry.isWriteAccess ? modes.DocumentHighlightKind.Write : modes.DocumentHighlightKind.Text
                    };
                });
            }));
        };
        return OccurrencesAdapter;
    }(Adapter));
    // --- definition ------
    var DefinitionAdapter = (function (_super) {
        __extends(DefinitionAdapter, _super);
        function DefinitionAdapter() {
            _super.apply(this, arguments);
        }
        DefinitionAdapter.prototype.provideDefinition = function (model, position, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getDefinitionAtPosition(resource.toString(), _this._positionToOffset(resource, position));
            }).then(function (entries) {
                if (!entries) {
                    return;
                }
                var result = [];
                for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                    var entry = entries_1[_i];
                    var uri = uri_1.default.parse(entry.fileName);
                    if (_this._modelService.getModel(uri)) {
                        result.push({
                            uri: uri,
                            range: _this._textSpanToRange(uri, entry.textSpan)
                        });
                    }
                }
                return result;
            }));
        };
        return DefinitionAdapter;
    }(Adapter));
    // --- references ------
    var ReferenceAdapter = (function (_super) {
        __extends(ReferenceAdapter, _super);
        function ReferenceAdapter() {
            _super.apply(this, arguments);
        }
        ReferenceAdapter.prototype.provideReferences = function (model, position, context, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getReferencesAtPosition(resource.toString(), _this._positionToOffset(resource, position));
            }).then(function (entries) {
                if (!entries) {
                    return;
                }
                var result = [];
                for (var _i = 0, entries_2 = entries; _i < entries_2.length; _i++) {
                    var entry = entries_2[_i];
                    var uri = uri_1.default.parse(entry.fileName);
                    if (_this._modelService.getModel(uri)) {
                        result.push({
                            uri: uri,
                            range: _this._textSpanToRange(uri, entry.textSpan)
                        });
                    }
                }
                return result;
            }));
        };
        return ReferenceAdapter;
    }(Adapter));
    // --- outline ------
    var OutlineAdapter = (function (_super) {
        __extends(OutlineAdapter, _super);
        function OutlineAdapter() {
            _super.apply(this, arguments);
        }
        OutlineAdapter.prototype.provideDocumentSymbols = function (model, token) {
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) { return worker.getNavigationBarItems(resource.toString()); }).then(function (items) {
                if (!items) {
                    return;
                }
                function convert(bucket, item, containerLabel) {
                    var result = {
                        name: item.text,
                        kind: outlineTypeTable[item.kind] || modes.SymbolKind.Variable,
                        location: {
                            uri: resource,
                            range: this._textSpanToRange(resource, item.spans[0])
                        },
                        containerName: containerLabel
                    };
                    if (item.childItems && item.childItems.length > 0) {
                        for (var _i = 0, _a = item.childItems; _i < _a.length; _i++) {
                            var child = _a[_i];
                            convert(bucket, child, result.name);
                        }
                    }
                    bucket.push(result);
                }
                var result = [];
                items.forEach(function (item) { return convert(result, item); });
                return result;
            }));
        };
        return OutlineAdapter;
    }(Adapter));
    var Kind = (function () {
        function Kind() {
        }
        Kind.unknown = '';
        Kind.keyword = 'keyword';
        Kind.script = 'script';
        Kind.module = 'module';
        Kind.class = 'class';
        Kind.interface = 'interface';
        Kind.type = 'type';
        Kind.enum = 'enum';
        Kind.variable = 'var';
        Kind.localVariable = 'local var';
        Kind.function = 'function';
        Kind.localFunction = 'local function';
        Kind.memberFunction = 'method';
        Kind.memberGetAccessor = 'getter';
        Kind.memberSetAccessor = 'setter';
        Kind.memberVariable = 'property';
        Kind.constructorImplementation = 'constructor';
        Kind.callSignature = 'call';
        Kind.indexSignature = 'index';
        Kind.constructSignature = 'construct';
        Kind.parameter = 'parameter';
        Kind.typeParameter = 'type parameter';
        Kind.primitiveType = 'primitive type';
        Kind.label = 'label';
        Kind.alias = 'alias';
        Kind.const = 'const';
        Kind.let = 'let';
        Kind.warning = 'warning';
        return Kind;
    }());
    exports.Kind = Kind;
    var outlineTypeTable = Object.create(null);
    outlineTypeTable[Kind.module] = modes.SymbolKind.Module;
    outlineTypeTable[Kind.class] = modes.SymbolKind.Class;
    outlineTypeTable[Kind.enum] = modes.SymbolKind.Enum;
    outlineTypeTable[Kind.interface] = modes.SymbolKind.Interface;
    outlineTypeTable[Kind.memberFunction] = modes.SymbolKind.Method;
    outlineTypeTable[Kind.memberVariable] = modes.SymbolKind.Property;
    outlineTypeTable[Kind.memberGetAccessor] = modes.SymbolKind.Property;
    outlineTypeTable[Kind.memberSetAccessor] = modes.SymbolKind.Property;
    outlineTypeTable[Kind.variable] = modes.SymbolKind.Variable;
    outlineTypeTable[Kind.const] = modes.SymbolKind.Variable;
    outlineTypeTable[Kind.localVariable] = modes.SymbolKind.Variable;
    outlineTypeTable[Kind.variable] = modes.SymbolKind.Variable;
    outlineTypeTable[Kind.function] = modes.SymbolKind.Function;
    outlineTypeTable[Kind.localFunction] = modes.SymbolKind.Function;
    // --- formatting ----
    var FormatHelper = (function (_super) {
        __extends(FormatHelper, _super);
        function FormatHelper() {
            _super.apply(this, arguments);
        }
        FormatHelper._convertOptions = function (options) {
            return {
                ConvertTabsToSpaces: options.insertSpaces,
                TabSize: options.tabSize,
                IndentSize: options.tabSize,
                IndentStyle: ts.IndentStyle.Smart,
                NewLineCharacter: '\n',
                InsertSpaceAfterCommaDelimiter: true,
                InsertSpaceAfterFunctionKeywordForAnonymousFunctions: false,
                InsertSpaceAfterKeywordsInControlFlowStatements: false,
                InsertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis: true,
                InsertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets: true,
                InsertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces: true,
                InsertSpaceAfterSemicolonInForStatements: false,
                InsertSpaceBeforeAndAfterBinaryOperators: true,
                PlaceOpenBraceOnNewLineForControlBlocks: false,
                PlaceOpenBraceOnNewLineForFunctions: false
            };
        };
        FormatHelper.prototype._convertTextChanges = function (resource, change) {
            return {
                text: change.newText,
                range: this._textSpanToRange(resource, change.span)
            };
        };
        return FormatHelper;
    }(Adapter));
    var FormatAdapter = (function (_super) {
        __extends(FormatAdapter, _super);
        function FormatAdapter() {
            _super.apply(this, arguments);
        }
        FormatAdapter.prototype.provideDocumentRangeFormattingEdits = function (model, range, options, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getFormattingEditsForRange(resource.toString(), _this._positionToOffset(resource, { lineNumber: range.startLineNumber, column: range.startColumn }), _this._positionToOffset(resource, { lineNumber: range.endLineNumber, column: range.endColumn }), FormatHelper._convertOptions(options));
            }).then(function (edits) {
                if (edits) {
                    return edits.map(function (edit) { return _this._convertTextChanges(resource, edit); });
                }
            }));
        };
        return FormatAdapter;
    }(FormatHelper));
    var FormatOnTypeAdapter = (function (_super) {
        __extends(FormatOnTypeAdapter, _super);
        function FormatOnTypeAdapter() {
            _super.apply(this, arguments);
        }
        Object.defineProperty(FormatOnTypeAdapter.prototype, "autoFormatTriggerCharacters", {
            get: function () {
                return [';', '}', '\n'];
            },
            enumerable: true,
            configurable: true
        });
        FormatOnTypeAdapter.prototype.provideOnTypeFormattingEdits = function (model, position, ch, options, token) {
            var _this = this;
            var resource = model.uri;
            return async_1.wireCancellationToken(token, this._worker(resource).then(function (worker) {
                return worker.getFormattingEditsAfterKeystroke(resource.toString(), _this._positionToOffset(resource, position), ch, FormatHelper._convertOptions(options));
            }).then(function (edits) {
                if (edits) {
                    return edits.map(function (edit) { return _this._convertTextChanges(resource, edit); });
                }
            }));
        };
        return FormatOnTypeAdapter;
    }(FormatHelper));
});

define("vs/languages/typescript/common/workerManager", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/worker/defaultWorkerFactory', 'vs/base/common/worker/simpleWorker', 'vs/editor/common/services/editorWorkerServiceImpl', './typescript'], function (require, exports, lifecycle_1, defaultWorkerFactory_1, simpleWorker_1, editorWorkerServiceImpl_1, typescript_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkerManager = (function () {
        function WorkerManager(modelService, defaults) {
            this._client = null;
            this._clientDispose = [];
            this._factory = new defaultWorkerFactory_1.DefaultWorkerFactory();
            this._modelService = modelService;
            this._defaults = defaults;
        }
        WorkerManager.prototype._createClient = function () {
            var _this = this;
            var client = new simpleWorker_1.SimpleWorkerClient(this._factory, 'vs/languages/typescript/common/worker', typescript_1.TypeScriptWorkerProtocol);
            var manager = new editorWorkerServiceImpl_1.EditorModelManager(client.get(), this._modelService, true);
            this._clientDispose.push(manager);
            this._clientDispose.push(client);
            var stopWorker = function () {
                _this._clientDispose = lifecycle_1.dispose(_this._clientDispose);
                _this._client = null;
            };
            // stop worker after being idle
            var handle = setInterval(function () {
                if (Date.now() - client.getLastRequestTimestamp() > 1000 * 60) {
                    stopWorker();
                }
            }, 1000 * 60);
            this._clientDispose.push({ dispose: function () { clearInterval(handle); } });
            // stop worker when defaults change
            this._clientDispose.push(this._defaults.onDidChange(function () { return stopWorker(); }));
            // send default to worker right away
            var worker = client.get();
            var _a = this._defaults, compilerOptions = _a.compilerOptions, extraLibs = _a.extraLibs;
            return worker.acceptDefaults(compilerOptions, extraLibs).then(function () { return ({ worker: worker, manager: manager }); });
        };
        WorkerManager.prototype.dispose = function () {
            this._clientDispose = lifecycle_1.dispose(this._clientDispose);
            this._client = null;
        };
        WorkerManager.prototype.getLanguageServiceWorker = function () {
            var resources = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                resources[_i - 0] = arguments[_i];
            }
            if (!this._client) {
                this._client = this._createClient();
            }
            return this._client
                .then(function (data) { return data.manager.withSyncedResources(resources)
                .then(function (_) { return data.worker; }); });
        };
        return WorkerManager;
    }());
    function create(defaults, modelService) {
        return new WorkerManager(modelService, defaults);
    }
    exports.create = create;
});

define("vs/languages/typescript/common/mode", ["require", "exports", 'vs/editor/common/modes', 'vs/languages/typescript/common/tokenization', 'vs/editor/common/modes/abstractMode', 'vs/editor/common/modes/supports/richEditSupport', 'vs/editor/common/services/modelService', 'vs/editor/common/services/modeService', 'vs/platform/markers/common/markers', './typescript', './languageFeatures', 'vs/languages/typescript/common/workerManager'], function (require, exports, modes, tokenization_1, abstractMode_1, richEditSupport_1, modelService_1, modeService_1, markers_1, typescript_1, languageFeatures_1, workerManager) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function setupMode(modelService, markerService, modeService, defaults, modeId, language) {
        var disposables = [];
        var client = workerManager.create(defaults, modelService);
        disposables.push(client);
        var registration = languageFeatures_1.register(modelService, markerService, modeId, defaults, function (first) {
            var more = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                more[_i - 1] = arguments[_i];
            }
            return client.getLanguageServiceWorker.apply(client, [first].concat(more));
        });
        disposables.push(registration);
        disposables.push(modeService.registerRichEditSupport(modeId, richEditConfiguration));
        disposables.push(modeService.registerTokenizationSupport2(modeId, tokenization_1.createTokenizationSupport2(language)));
    }
    var richEditConfiguration = {
        wordPattern: abstractMode_1.createWordRegExp('$'),
        comments: {
            lineComment: '//',
            blockComment: ['/*', '*/']
        },
        brackets: [
            ['{', '}'],
            ['[', ']'],
            ['(', ')']
        ],
        onEnterRules: [
            {
                // e.g. /** | */
                beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
                afterText: /^\s*\*\/$/,
                action: { indentAction: modes.IndentAction.IndentOutdent, appendText: ' * ' }
            },
            {
                // e.g. /** ...|
                beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
                action: { indentAction: modes.IndentAction.None, appendText: ' * ' }
            },
            {
                // e.g.  * ...|
                beforeText: /^(\t|(\ \ ))*\ \*(\ ([^\*]|\*(?!\/))*)?$/,
                action: { indentAction: modes.IndentAction.None, appendText: '* ' }
            },
            {
                // e.g.  */|
                beforeText: /^(\t|(\ \ ))*\ \*\/\s*$/,
                action: { indentAction: modes.IndentAction.None, removeText: 1 }
            },
            {
                // e.g.  *-----*/|
                beforeText: /^(\t|(\ \ ))*\ \*[^/]*\*\/\s*$/,
                action: { indentAction: modes.IndentAction.None, removeText: 1 }
            }
        ],
        __electricCharacterSupport: {
            docComment: { scope: 'comment.doc', open: '/**', lineStart: ' * ', close: ' */' }
        },
        autoClosingPairs: [
            { open: '{', close: '}' },
            { open: '[', close: ']' },
            { open: '(', close: ')' },
            { open: '"', close: '"', notIn: ['string'] },
            { open: '\'', close: '\'', notIn: ['string', 'comment'] },
            { open: '`', close: '`' }
        ]
    };
    function createRichEditSupport(modeId) {
        return new richEditSupport_1.RichEditSupport(modeId, null, richEditConfiguration);
    }
    exports.createRichEditSupport = createRichEditSupport;
    var isActivated = false;
    function activate(ctx) {
        if (isActivated) {
            return;
        }
        isActivated = true;
        var modelService = ctx.get(modelService_1.IModelService);
        var markerService = ctx.get(markers_1.IMarkerService);
        var modeService = ctx.get(modeService_1.IModeService);
        setupMode(modelService, markerService, modeService, typescript_1.typeScriptDefaults, 'typescript', tokenization_1.Language.TypeScript);
        setupMode(modelService, markerService, modeService, typescript_1.javaScriptDefaults, 'javascript', tokenization_1.Language.EcmaScript5);
    }
    exports.activate = activate;
});

//# sourceMappingURL=mode.js.map
